@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2021, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
@@ -223,6 +223,7 @@ function deprecate ({ dbPassword, dbUser, ssl, sslOptions }) {
 }
 
 /**
+ * @private
  * @alias module:Connection
  * @param {Properties} - connection properties
  * @returns {module:Connection}
@@ -660,7 +661,7 @@ function Connection ({ auth, connectionAttributes = {}, connectTimeout = 10000,
                         // When a connection becomes unavailable, it should
                         // include the timestamp of when it was last tried.
                         const unavailable = Object.assign({}, state.endpoints.available.shift(), {
-                            unavailableAt: Date.now()
+                            unavailableAt: system.time()
                         });
 
                         state.endpoints.unavailable.push(unavailable);
@@ -1237,7 +1238,7 @@ function Connection ({ auth, connectionAttributes = {}, connectTimeout = 10000,
          * @returns {module:Connection}
          */
         update () {
-            const now = Date.now();
+            const now = system.time();
             // Check which unavailable endpoints can be re-tried.
             // If the current element in the list is not "retryable", neither
             // are the remaining. A retryable endpoint is any endpoint that has
