@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2020, 2022, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
@@ -29,7 +29,8 @@
  */
 
 /**
- * Custom application-level error messages
+ * Custom application-level error messages.
+ * @private
  */
 
 exports.MESSAGES = {
@@ -64,9 +65,9 @@ exports.MESSAGES = {
     ER_DEVAPI_BAD_TLS_CA_PATH: 'The certificate authority (CA) file path is not valid.',
     ER_DEVAPI_BAD_TLS_CRL_PATH: 'The certificate revocation list (CRL) file path is not valid.',
     ER_DEVAPI_BAD_TLS_CIPHERSUITE_LIST: '%s is not a valid TLS ciphersuite list format.',
-    ER_DEVAPI_BAD_TLS_OPTIONS: 'Additional TLS options cannot be specified when TLS is disabled.',
     ER_DEVAPI_BAD_TLS_VERSION: '"%s" is not a valid TLS protocol version. Should be one of %s.',
     ER_DEVAPI_BAD_TLS_VERSION_LIST: '"%s" is not a valid TLS protocol list format.',
+    ER_DEVAPI_CERTIFICATE_AUTHORITY_REQUIRED: '%s requires a certificate authority.',
     ER_DEVAPI_COLLECTION_OPTIONS_NOT_SUPPORTED: 'Your MySQL server does not support the requested operation. Please update to MySQL 8.0.19 or a later version.',
     ER_DEVAPI_CONNECTION_CLOSED: 'This session was closed. Use "mysqlx.getSession()" or "mysqlx.getClient()" to create a new one.',
     ER_DEVAPI_CONNECTION_TIMEOUT: 'Connection attempt to the server was aborted. Timeout of %d ms was exceeded.',
@@ -105,6 +106,7 @@ exports.MESSAGES = {
 /**
  * MySQL Server errors.
  * https://dev.mysql.com/doc/mysql-errors/8.0/en/server-error-reference.html
+ * @private
  */
 
 exports.ER_DB_CREATE_EXISTS = 1007;
@@ -130,6 +132,7 @@ exports.ER_LOCK_NOWAIT = 3572;
 
 /**
  * X DevAPI errors.
+ * @private
  */
 
 exports.ER_DEVAPI_MULTI_HOST_CONNECTION_FAILED = 4001;
@@ -137,6 +140,7 @@ exports.ER_DEVAPI_MULTI_HOST_CONNECTION_FAILED = 4001;
 /**
  * X Plugin errors.
  * https://github.com/mysql/mysql-server/blob/8.0/plugin/x/src/xpl_error.h
+ * @private
  */
 
 exports.ER_X_CAPABILITIES_PREPARE_FAILED = 5001;
@@ -157,6 +161,7 @@ exports.ER_X_INVALID_VALIDATION_SCHEMA = 5182;
 
 /**
  * Global error mapping.
+ * @private
  */
 
 exports[1007] = 'ER_DB_CREATE_EXISTS';
